Rem Copyright (c) 2014, Oracle and/or its affiliates. All rights reserved.
Rem
Rem    NAME
Rem      configure_oc4hadoop.sql
Rem
Rem    DESCRIPTION
Rem     Create and configure OC4H_INSTALL schema for oc4hadoop installation.
Rem
Rem    VERSION
Rem     1.0.1
Rem

create or replace package oc4hadoop_sys authid current_user as
 procedure require_sys;
 procedure create_user;
 procedure create_role;
 procedure configure;
 procedure load_jar(filename varchar2, force boolean);
 procedure load_jar(filename varchar2);
 procedure drop_jar(filename varchar2);
 procedure export_class(name varchar2);
 procedure export_resource(name varchar2);
end oc4hadoop_sys;
/ 

create or replace package body oc4hadoop_sys as

install_schema  VARCHAR2(20) := 'OC4H_INSTALL';

procedure require_sys as
begin
  if login_user != 'SYS' then
    declare
      foo exception;
      pragma exception_init(foo,-1031);
    begin
      raise foo;
    end;
  end if;
end;

procedure create_user as
  user_exists integer := 0;
begin
  select count(1) into user_exists from dba_users where username = install_schema;
  if user_exists != 0 then
    raise_application_error(-20000,'User ' || install_schema || ' already exists !! If upgrading oc4hadoop then first run uninstall_oc4hadoop.sql');    
  else
    execute immediate 'create user ' || install_schema || ' identified by &1';
    dbms_java.grant_permission( install_schema , 'SYS:java.lang.RuntimePermission', 'getClassLoader', null);
    dbms_java.grant_permission( install_schema , 'SYS:java.lang.RuntimePermission', 'setContextClassLoader', '');
    dbms_java.grant_permission( install_schema , 'SYS:java.lang.RuntimePermission', 'accessDeclaredMembers', null);
    dbms_java.grant_permission( install_schema , 'SYS:java.lang.reflect.ReflectPermission', 'suppressAccessChecks', null);
    -- For Client Installation 
    initjvmaux.exec('grant create session to OC4H_INSTALL');
    initjvmaux.exec('grant create table to OC4H_INSTALL');
    initjvmaux.exec('alter user OC4H_INSTALL quota 10m on system');
    initjvmaux.exec('grant resource to OC4H_INSTALL');
    dbms_java.grant_permission( 'OC4H_INSTALL', 'SYS:oracle.aurora.security.JServerPermission', 'Verifier', '' );
  end if;
end;

procedure load_jar(filename varchar2, force boolean) as
  verifier_status number;
  OC4HADOOP_HOME varchar2(100);
  trash varchar2(300) := dbms_java.endsession_and_related_state;
begin
  require_sys;
  verifier_status := dbms_java.option_controller(2,3);
  begin
    DBMS_SYSTEM.get_env ('OC4HADOOP_HOME', OC4HADOOP_HOME);
    if OC4HADOOP_HOME is null then
      OC4HADOOP_HOME := 'oc4hadoop';
    end if;
    if force then
      dbms_java.loadjava('-v -force -schema ' || install_schema || ' -resolver ((* ' || install_schema || ')(* SYS)(* -)) -resolve '||OC4HADOOP_HOME||'/'||filename);
    else
      dbms_java.loadjava('-v -schema ' || install_schema || ' -resolver ((* ' || install_schema || ')(* SYS)(* -)) -resolve '||OC4HADOOP_HOME||'/'||filename);
    end if;
    if verifier_status = 1 then
       verifier_status := dbms_java.option_controller(2,2);
    end if;
  exception when others then
    if verifier_status = 1 then
       verifier_status := dbms_java.option_controller(2,2);
    end if;
    raise;
  end;
end;

procedure load_jar(filename varchar2) as
begin
  load_jar(filename,true);
end;

procedure drop_jar(filename varchar2) as
  verifier_status number;
  OC4HADOOP_HOME varchar2(100);
  trash varchar2(300) := dbms_java.endsession_and_related_state;
begin
  require_sys;
  verifier_status := dbms_java.option_controller(2,3);
  begin
    DBMS_SYSTEM.get_env ('OC4HADOOP_HOME', OC4HADOOP_HOME);
    if OC4HADOOP_HOME is null then
      OC4HADOOP_HOME := 'oc4hadoop';
    end if;
    DBMS_OUTPUT.PUT_LINE('Dropping '||filename);
    dbms_java.dropjava('-v -schema ' || install_schema || ' -synonym '||OC4HADOOP_HOME||'/'||filename);
    if verifier_status = 1 then
       verifier_status := dbms_java.option_controller(2,2);
    end if;
  exception when others then
    if verifier_status = 1 then
       verifier_status := dbms_java.option_controller(2,2);
    end if;
    raise;
  end;
end;


procedure create_role as
begin
  require_sys;
  initjvmaux.drp('drop role oc4hadoop');
  initjvmaux.exec('create role oc4hadoop');
  initjvmaux.exec('grant oc4hadoop to SYS');
  dbms_java.grant_permission('OC4HADOOP', 'SYS:java.lang.RuntimePermission', 'getenv.*', '');
  -- grant oc4hadoop-specific guarding permisssions
  dbms_java.grant_permission( 'OC4HADOOP', 'SYS:java.lang.RuntimePermission', 'oc4hadoopUser', '' );  
  dbms_java.grant_permission( 'OC4HADOOP', 'SYS:java.lang.RuntimePermission', 'oc4hadoopFindMethod', '' );  
  dbms_java.grant_permission( 'OC4HADOOP', 'SYS:java.lang.RuntimePermission', 'oc4hadoopFindField', '' );  
  execute immediate 'grant oc4hadoop to '|| install_schema;
end;

procedure configure as 
begin
  create_user;
  create_role;
end;

procedure export_class(name varchar2) as
  id varchar2(256);
begin
  require_sys;
  id := sys.dbms_assert.enquote_name(name, false)  ;
  execute immediate 'grant execute on ' || install_schema || '.' || id || ' to public';
  execute immediate 'create or replace public synonym ' || id || ' for ' || 
     install_schema || '.' || id;
end;

procedure export_resource(name varchar2) as
  id varchar2(256);
begin
  require_sys;
  id := sys.dbms_assert.enquote_name(name, false)  ;
  execute immediate 'grant execute on java resource ' || install_schema || '.' || id || ' to public';
  execute immediate 'create or replace public synonym ' || id || ' for ' || 
     install_schema || '.' || id;
end;
end oc4hadoop_sys;
/

create or replace package oc4hadoop_util authid current_user as
 procedure app_loader( filename varchar2 );
end oc4hadoop_util;
/

create or replace package body oc4hadoop_util as
OC4HADOOP_HOME varchar2(100);
schema varchar2(100);
procedure app_loader( filename varchar2 ) as
begin
  DBMS_SYSTEM.get_env ('OC4HADOOP_HOME', OC4HADOOP_HOME);
    if OC4HADOOP_HOME is null then
      OC4HADOOP_HOME := 'oc4hadoop';
    end if;
   SELECT USER INTO schema FROM dual;
   dbms_java.loadjava('-v -force -resolver ((* '|| schema ||')(* OC4H_INSTALL)(* SYS)) -resolve '||OC4HADOOP_HOME||'/'||filename);
end;
end oc4hadoop_util;
/

CREATE OR REPLACE PUBLIC SYNONYM oc4hadoop_util FOR oc4hadoop_util;
GRANT EXECUTE ON oc4hadoop_util TO PUBLIC;

call javavm_sys.uninstall_dbhadoop();
call oc4hadoop_sys.configure();
