/* Copyright (c) 2011, 2014, Oracle and/or its affiliates. 
All rights reserved.*/

/*
   DESCRIPTION
    The example code for inverted index on text. The input is in the form of
    <LongWritable, Text>. The output is in the form of <Text, TextArrayWritable>.
    Some stop words and irregular ones in the text are ignored.

   VERSION
    1.0.1
 */

package oracle.sql.hadoop.examples;

import java.io.IOException;
import java.util.StringTokenizer;
import java.util.HashSet;
import java.util.Set;
import java.util.ArrayList;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.io.Writable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.ArrayWritable;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.Reducer;
import org.apache.hadoop.mapreduce.lib.input.FileSplit;

import oracle.sql.hadoop.ConfigManager;
import oracle.sql.hadoop.Job;

public class InvertedIndex {

  public static class TextArrayWritable extends ArrayWritable {
    public TextArrayWritable() {
      super(Text.class);
    }
  }

  /**
   * Parse text from table and output <word, pos@doc> pairs
   */
  public static class InvertedIndexMapper
    extends Mapper<LongWritable, Text, Text, Text> {

    private static Set<String> googleStopwords;

    static {
      googleStopwords = new HashSet<String>();
      googleStopwords.add("I");
      googleStopwords.add("a");
      googleStopwords.add("about");
      googleStopwords.add("an");
      googleStopwords.add("are");
      googleStopwords.add("as");
      googleStopwords.add("at");
      googleStopwords.add("be");
      googleStopwords.add("by");
      googleStopwords.add("com");
      googleStopwords.add("de");
      googleStopwords.add("en");
      googleStopwords.add("for");
      googleStopwords.add("from");
      googleStopwords.add("how");
      googleStopwords.add("in");
      googleStopwords.add("is");
      googleStopwords.add("it");
      googleStopwords.add("la");
      googleStopwords.add("of");
      googleStopwords.add("on");
      googleStopwords.add("or");
      googleStopwords.add("that");
      googleStopwords.add("the");
      googleStopwords.add("this");
      googleStopwords.add("to");
      googleStopwords.add("was");
      googleStopwords.add("what");
      googleStopwords.add("when");
      googleStopwords.add("where");
      googleStopwords.add("who");
      googleStopwords.add("will");
      googleStopwords.add("with");
      googleStopwords.add("and");
      googleStopwords.add("the");
      googleStopwords.add("www");
    }

    private Text word = new Text();
    private Text posAtDoc = new Text();

    public void map(LongWritable key, Text value, Context context)
      throws IOException, InterruptedException {

      String doc = ((FileSplit) (context.getInputSplit())).getPath()
        .getName();
      StringTokenizer itr = new StringTokenizer(value.toString());
      while (itr.hasMoreTokens()) {
        String matchedKey = itr.nextToken().toLowerCase();
        int len = matchedKey.length();
        // Remove irregular and stop words.
        if (!Character.isLetter(matchedKey.charAt(0))
            || Character.isDigit(matchedKey.charAt(0))
            || googleStopwords.contains(matchedKey)
            || matchedKey.contains("_") || matchedKey.contains("-")
            || matchedKey.contains(".")
            || matchedKey.length() >= 50) {
          continue;
        }
        // Remove tailing non-letter characters.
        while (!Character.isLetter(matchedKey.charAt(len - 1))) {
          --len;
        }
        word.set(matchedKey.substring(0, len));
        posAtDoc.set(key + "@" + doc);
        context.write(word, posAtDoc);
      }
    }
  }

  public static class InvertedIndexReducer
    extends Reducer<Text, Text, Text, TextArrayWritable> {

    public void reduce(Text key, Iterable<Text> values, Context context)
      throws IOException, InterruptedException {
      TextArrayWritable value = new TextArrayWritable();
      ArrayList<Text> list = new ArrayList<Text>();
      for (Text val : values) {
        list.add(val);
      }
      Writable[] array = new Writable[list.size()];
      list.toArray(array);
      value.set(array);
      context.write(key, value);
    }
  }

  public static void run(String arg) throws Exception {
    run(arg, 0);
  }

  public static void run(String arg, int n)
    throws Exception {
    // Take the user specified options and pass them to the parser.
    Configuration conf = new Configuration();
    ConfigManager.parse(conf, arg);
    // Specify the job parameters.
    Job job = new Job(conf, "InvertedIndex");
    job.setMapperClass(oracle.sql.hadoop.examples.InvertedIndex.InvertedIndexMapper.class);
    job.setReducerClass(oracle.sql.hadoop.examples.InvertedIndex.InvertedIndexReducer.class);
    job.setInputKeyClass(LongWritable.class);
    job.setInputValueClass(Text.class);
    job.setMapOutputKeyClass(Text.class);
    job.setMapOutputValueClass(Text.class);
    job.setOutputKeyClass(Text.class);
    job.setOutputValueClass(oracle.sql.hadoop.examples.InvertedIndex.TextArrayWritable.class);

    job.setMapOutputKeyDBType("VARCHAR2(50)");
    job.setMapOutputValueDBType("VARCHAR2(200)");
    job.setOutputKeyDBType("VARCHAR2(50)");
    job.setOutputValueDBType("ARRAYLIST");
    if (n > 0) {
      job.setNumMapTasks(n);
      job.setNumReduceTasks(n);
    }
    job.init();
    job.run("InTable", "OutTable");
  }
}
