/* Copyright (c) 2014, Oracle and/or its affiliates. All rights reserved.*/

/*
   DESCRIPTION
    This demo takes input from two tables which have profit values for 
    different products in different currencies (euro & yen) and genrates
    output table with total profit in usd currency.

   VERSION
    1.0.1
 */

package oracle.sql.hadoop.examples;

import java.io.IOException;
import oracle.sql.hadoop.DBMultipleInputs;
import oracle.sql.hadoop.Job;
import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.Reducer;

class ProfitCalculator {

  private final static double EURO_FACTOR = 1.39;
  private final static double YEN_FACTOR = 0.0098;

  public static class EuroMapper
    extends Mapper<IntWritable, DoubleWritable, IntWritable, DoubleWritable> {
    private DoubleWritable profit = new DoubleWritable();
    @Override
    public void map(IntWritable Key, DoubleWritable value,
            Context context) throws IOException, InterruptedException {
      profit.set(value.get()*EURO_FACTOR);
      context.write(Key, profit);      
    }
  }

  public static class YenMapper
    extends Mapper<IntWritable, DoubleWritable, IntWritable, DoubleWritable> {
    private DoubleWritable profit = new DoubleWritable();
    @Override
    public void map(IntWritable Key, DoubleWritable value,
            Context context) throws IOException, InterruptedException {
      profit.set(value.get()*YEN_FACTOR);
      context.write(Key, profit);      
    }
  }

  public static class DoubleSumReducer
    extends Reducer<IntWritable, DoubleWritable, IntWritable, DoubleWritable> {
    private DoubleWritable result = new DoubleWritable();
    @Override
    public void reduce(IntWritable Key, Iterable<DoubleWritable> values,
      Context context) throws IOException, InterruptedException {
      double sum = 0;
      for (DoubleWritable val: values) {
        sum += val.get();
      }
      result.set(sum);
      context.write(Key, result);
    }
  }

  public static void run() throws Exception {
    Job job = new Job();
    job.setJobName("ProfitCalculator");
    job.setInputKeyClass(IntWritable.class);
    job.setInputValueClass(DoubleWritable.class);
    job.setOutputKeyClass(IntWritable.class);
    job.setOutputValueClass(DoubleWritable.class);
    job.setOutputKeyDBType("INTEGER");
    job.setOutputValueDBType("NUMBER");
    job.setOutputTableName("OutTable");
    job.setReducerClass(DoubleSumReducer.class);
    DBMultipleInputs.addInputTable(job, "EuroProfitTable", EuroMapper.class);
    DBMultipleInputs.addInputTable(job, "YenProfitTable", YenMapper.class);
    job.init();
    job.run();
  }
}
