/* Copyright (c) 2011, 2014, Oracle and/or its affiliates. 
All rights reserved.*/

/*
DESCRIPTION
 Given records of <key, value> pairs, identify different user sessions.

VERSION
 1.0.1
 */

package oracle.sql.hadoop.examples;

import java.io.IOException;
import java.util.Collections;
import java.util.ArrayList;
import java.util.List;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.Reducer;

import oracle.sql.hadoop.ConfigManager;
import oracle.sql.hadoop.Job;

public class Sessionization
{

  private static final int TIMEOUT = 10;

  public static class SessionizationMapper extends Mapper<Text, LongWritable,
                                                          Text, LongWritable>
  {

    public void map(Text key, LongWritable value, Context context)
           throws IOException, InterruptedException
    {
      // Collect all values into a buffer
      context.write(key, value);
    }
  }

  public static class SessionizationReducer extends Reducer<Text, LongWritable,
                                                            Text, LongWritable>
  {

    private static int sessionid = 1;

    public void reduce(Text key, Iterable<LongWritable> values, Context context)
           throws IOException, InterruptedException
    {

      List<Long> buffer = new ArrayList<Long>();
      Text sessionKey = new Text();
      for (LongWritable value : values) {
        buffer.add(new Long(value.get()));
      }
      // Sort buffer by timestamp
      Collections.sort(buffer);

      long previous = Long.MIN_VALUE;
      for (Long value : buffer) {
        if (value - previous > TIMEOUT) {
          sessionid++;
        }

        previous = value;
        sessionKey.set(key.toString() + sessionid);
        context.write(sessionKey, new LongWritable(value));
      }
    }
  }

  public static void run(String arg) throws Exception
  {
    // Take the user specified options and pass them to the parser.
    Configuration conf = new Configuration();
    ConfigManager.parse(conf, arg);

    // Specify the job parameters.
    Job job = new Job(conf, "Sessionization");
    job.setMapperClass
        (oracle.sql.hadoop.examples.Sessionization.SessionizationMapper.class);
    job.setReducerClass
        (oracle.sql.hadoop.examples.Sessionization.SessionizationReducer.class);
    job.setInputKeyClass(Text.class);
    job.setInputValueClass(LongWritable.class);
    job.setOutputKeyClass(Text.class);
    job.setOutputValueClass(LongWritable.class);

    job.setMapOutputKeyDBType("VARCHAR2(10)");
    job.setMapOutputValueDBType("NUMBER");
    job.setOutputKeyDBType("VARCHAR2(10)");
    job.setOutputValueDBType("NUMBER");
    job.setNumMapTasks(16);
    job.setNumReduceTasks(16);
    job.init();
    job.run("InTable", "OutTable");
  }
}
